﻿using Hotline.Share.Dtos.FlowEngine;

namespace Hotline.FlowEngine.Definitions
{
    public interface IDefinitionDomainService
    {
        Task<string> AddAsync(AddDefinitionDto dto, CancellationToken cancellationToken);

        /// <summary>
        /// 发布流程模板
        /// </summary>
        Task PublishAsync(string id, CancellationToken cancellationToken);

        /// <summary>
        /// 发布（保存并发布）
        /// </summary>
        Task PublishAsync(AddDefinitionDto dto, CancellationToken cancellationToken);

        /// <summary>
        /// 查询最大版本号（含启用、禁用）
        /// </summary>
        /// <param name="code"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        Task<int> GetLastVersionAsync(string code, CancellationToken cancellationToken);

        /// <summary>
        /// 查询已启用的最后版本
        /// </summary>
        /// <param name="code"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        Task<Definition?> GetLastEnableAsync(string code, CancellationToken cancellationToken);

        /// <summary>
        /// 查询已启用的最后版本
        /// </summary>
        /// <param name="moduleCode"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        Task<Definition?> GetLastEnableByModuleCodeAsync(string moduleCode, CancellationToken cancellationToken);

        /// <summary>
        /// 禁用指定业务模块下所有模板
        /// </summary>
        Task EnableAsync(string moduleCode, CancellationToken cancellationToken);

        /// <summary>
        /// 未开启的流程，查找start节点配置的nextSteps信息，用作开启流程的传参
        /// </summary>
        /// <param name="definitionCode"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        Task<IReadOnlyList<StepDefine>> GetSecondStepsAsync(string definitionCode, CancellationToken cancellationToken);
    }
}
